@@ -9,6 +9,9 @@ logger = logging.getLogger(__name__)
 import json
 import base64
 
+from netaddr import IPAddress, IPNetwork
+from netaddr.core import AddrFormatError
+
 from django.http import HttpResponse
 from django.conf import settings
 from django.views.generic.base import View
@@ -19,8 +22,10 @@ from django.utils.decorators import method_decorator
 
 from ..utils import log, ddns_client
 from ..main.models import Host
-from ..main.dnstools import (update, delete, check_ip, put_ip_into_session,
-                             SameIpError, DnsUpdateError, NameServerNotAvailable)
+from ..main.dnstools import (FQDN, update, delete, check_ip, put_ip_into_session,
+                             SameIpError, DnsUpdateError, NameServerNotAvailable,
+                             Timeout)
+from ..main.iptools import normalize_ip
 
 
 def Response(content):
@@ -43,7 +48,7 @@ def myip_view(request, logger=None):
     """
     # Note: keeping this as a function-based view, as it is frequently used -
     # maybe it is slightly more efficient than class-based.
-    ipaddr = request.META['REMOTE_ADDR']
+    ipaddr = normalize_ip(request.META['REMOTE_ADDR'])
     logger.debug("detected remote ip address: %s" % ipaddr)
     return Response(ipaddr)
 
@@ -63,7 +68,7 @@ class DetectIpView(View):
         # so the session cookie is not received here - thus we access it via
         # the sessionid:
         s = SessionStore(session_key=sessionid)
-        ipaddr = request.META['REMOTE_ADDR']
+        ipaddr = normalize_ip(request.META['REMOTE_ADDR'])
         # as this is NOT the session automatically established and
         # also saved by the framework, we need to use save=True here
         put_ip_into_session(s, ipaddr, save=True)
@@ -223,7 +228,7 @@ class NicUpdateView(View):
             return Response('badagent')
         ipaddr = request.GET.get('myip')
         if not ipaddr:  # None or ''
-            ipaddr = request.META.get('REMOTE_ADDR')
+            ipaddr = normalize_ip(request.META.get('REMOTE_ADDR'))
         secure = request.is_secure()
         if delete:
             return _delete(host, ipaddr, secure, logger=logger)
@@ -281,7 +286,7 @@ class AuthorizedNicUpdateView(View):
         # and logged-in usage - thus misbehaved user agents are no problem.
         ipaddr = request.GET.get('myip')
         if not ipaddr:  # None or empty string
-            ipaddr = request.META.get('REMOTE_ADDR')
+            ipaddr = normalize_ip(request.META.get('REMOTE_ADDR'))
         secure = request.is_secure()
         if delete:
             return _delete(host, ipaddr, secure, logger=logger)
@@ -343,9 +348,56 @@ def _update(host, ipaddr, secure=False, logger=None):
     host.poke(kind, secure)
     try:
         update(fqdn, ipaddr)
+    except SameIpError:
+        msg = '%s - received no-change update, ip: %s tls: %r' % (fqdn, ipaddr, secure)
+        logger.warning(msg)
+        host.register_client_result(msg, fault=True)
+        return Response('nochg %s' % ipaddr)
+    except (DnsUpdateError, NameServerNotAvailable, Timeout) as e:
+        msg = str(e)
+        msg = '%s - received update that resulted in a dns error [%s], ip: %s tls: %r' % (
+            fqdn, msg, ipaddr, secure)
+        logger.error(msg)
+        host.register_server_result(msg, fault=True)
+        return Response('dnserr')
+    else:
         msg = '%s - received good update -> ip: %s tls: %r' % (fqdn, ipaddr, secure)
         logger.info(msg)
         host.register_client_result(msg, fault=False)
+        # update related hosts
+        for rh in host.relatedhosts.all():
+            if rh.available:
+                if kind == 'ipv4':
+                    ifid = rh.interface_id_ipv4.strip()
+                    netmask = host.netmask_ipv4
+                else:  # kind == 'ipv6':
+                    ifid = rh.interface_id_ipv6.strip()
+                    netmask = host.netmask_ipv6
+                if not ifid:
+                    # ifid can be just left blank if no address record of this type is wanted
+                    continue
+                try:
+                    ifid = IPAddress(ifid)
+                    network = IPNetwork("%s/%d" % (ipaddr, netmask))
+                    rh_ipaddr = str(IPAddress(network.network) + int(ifid))
+                    rh_fqdn = FQDN(rh.name + '.' + fqdn.host, fqdn.domain)
+                except AddrFormatError as e:
+                    logger.warning("trouble computing address of related host %s [%s]" % (rh, e))
+                else:
+                    logger.info("updating related host %s -> %s" % (rh_fqdn, rh_ipaddr))
+                    try:
+                        update(rh_fqdn, rh_ipaddr)
+                    except SameIpError:
+                        msg = '%s - received no-change update, ip: %s tls: %r' % (rh_fqdn, rh_ipaddr, secure)
+                        logger.warning(msg)
+                        host.register_client_result(msg, fault=True)
+                    except (DnsUpdateError, NameServerNotAvailable, Timeout) as e:
+                        msg = str(e)
+                        msg = '%s - received update that resulted in a dns error [%s], ip: %s tls: %r' % (
+                            rh_fqdn, msg, rh_ipaddr, secure)
+                        logger.error(msg)
+                        host.register_server_result(msg, fault=True)
+
         # now check if there are other services we shall relay updates to:
         for hc in host.serviceupdaterhostconfigs.all():
             if (kind == 'ipv4' and hc.give_ipv4 and hc.service.accept_ipv4
@@ -363,18 +415,6 @@ def _update(host, ipaddr, secure=False, logger=None):
                     kwargs.pop('password')
                     logger.exception("the dyndns2 updater raised an exception [%r]" % kwargs)
         return Response('good %s' % ipaddr)
-    except SameIpError:
-        msg = '%s - received no-change update, ip: %s tls: %r' % (fqdn, ipaddr, secure)
-        logger.warning(msg)
-        host.register_client_result(msg, fault=True)
-        return Response('nochg %s' % ipaddr)
-    except (DnsUpdateError, NameServerNotAvailable) as e:
-        msg = str(e)
-        msg = '%s - received update that resulted in a dns error [%s], ip: %s tls: %r' % (
-            fqdn, msg, ipaddr, secure)
-        logger.error(msg)
-        host.register_server_result(msg, fault=True)
-        return Response('dnserr')
 
 
 def _delete(host, ipaddr, secure=False, logger=None):
@@ -424,7 +464,7 @@ def _delete(host, ipaddr, secure=False, logger=None):
         host.register_client_result(msg, fault=False)
         # XXX unclear what to do for "other services" we relay updates to
         return Response('deleted %s' % rdtype)
-    except (DnsUpdateError, NameServerNotAvailable) as e:
+    except (DnsUpdateError, NameServerNotAvailable, Timeout) as e:
         msg = str(e)
         msg = '%s - received delete for record %s that resulted in a dns error [%s], tls: %r' % (
             fqdn, rdtype, msg, secure)
