@@ -31,8 +31,8 @@ def result_fmt(msg):
     return msg[:RESULT_MSG_LEN]
 
 
-class BlacklistedDomain(models.Model):
-    domain = models.CharField(
+class BlacklistedHost(models.Model):
+    name_re = models.CharField(
         max_length=255,
         unique=True,
         help_text=_('Blacklisted domain. Evaluated as regex (search).'))
@@ -42,12 +42,12 @@ class BlacklistedDomain(models.Model):
     created_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='blacklisted_domains')
 
     def __unicode__(self):
-        return u"%s" % (self.domain, )
+        return u"%s" % (self.name_re, )
 
 
-def domain_blacklist_validator(value):
-    for bd in BlacklistedDomain.objects.all():
-        if re.search(bd.domain, value):
+def host_blacklist_validator(value):
+    for bd in BlacklistedHost.objects.all():
+        if re.search(bd.name_re, value):
             raise ValidationError(u'This name is blacklisted')
 
 
@@ -69,7 +69,7 @@ UPDATE_ALGORITHM_CHOICES = [(k, k) for k in UPDATE_ALGORITHMS]
 
 
 class Domain(models.Model):
-    domain = models.CharField(
+    name = models.CharField(
         max_length=255,  # RFC 2181 (and also: max length of unique fields)
         unique=True,
         help_text=_("Name of the zone where dynamic hosts may get added"))
@@ -105,7 +105,7 @@ class Domain(models.Model):
     created_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='domains')
 
     def __unicode__(self):
-        return u"%s" % (self.domain, )
+        return u"%s" % (self.name, )
 
     def generate_ns_secret(self):
         algorithm = self.nameserver_update_algorithm
@@ -122,14 +122,14 @@ class Domain(models.Model):
 
 
 class Host(models.Model):
-    subdomain = models.CharField(
+    name = models.CharField(
         max_length=255,  # RFC 2181 (and considering having multiple joined labels here later)
         validators=[
             RegexValidator(
                 regex=r'^(([a-z0-9][a-z0-9\-]*[a-z0-9])|[a-z0-9])$',
-                message='Invalid subdomain: only "a-z", "0-9" and "-" is allowed'
+                message='Invalid host name: only "a-z", "0-9" and "-" is allowed'
             ),
-            domain_blacklist_validator,
+            host_blacklist_validator,
         ],
         help_text=_("The name of your host."))
     domain = models.ForeignKey(Domain, on_delete=models.CASCADE)
@@ -148,7 +148,12 @@ class Host(models.Model):
         default=True,
         help_text=_("Check if host is available/in use - "
                     "if not checked, we won't accept updates for this host"))
-
+    netmask_ipv4 = models.PositiveSmallIntegerField(
+        default=32,
+        help_text=_("Netmask/Prefix length for IPv4."))
+    netmask_ipv6 = models.PositiveSmallIntegerField(
+        default=64,
+        help_text=_("Netmask/Prefix length for IPv6."))
     # abuse means that we (either the operator or some automatic mechanism)
     # think the host is used in some abusive or unfair way, e.g.:
     # sending nochg updates way too often or otherwise using a defect,
@@ -194,14 +199,14 @@ class Host(models.Model):
 
     def __unicode__(self):
         return u"%s.%s" % (
-            self.subdomain, self.domain.domain)
+            self.name, self.domain.name)
 
     class Meta(object):
-        unique_together = (('subdomain', 'domain'), )
-        index_together = (('subdomain', 'domain'), )
+        unique_together = (('name', 'domain'), )
+        index_together = (('name', 'domain'), )
 
     def get_fqdn(self):
-        return dnstools.FQDN(self.subdomain, self.domain.domain)
+        return dnstools.FQDN(self.name, self.domain.name)
 
     @classmethod
     def get_by_fqdn(cls, fqdn, **kwargs):
@@ -210,7 +215,7 @@ class Host(models.Model):
         if len(splitted) != 2:
             raise ValueError("get_by_fqdn(%s): FQDN has to contain (at least) one dot" % fqdn)
         try:
-            host = Host.objects.get(subdomain=splitted[0], domain__domain=splitted[1], **kwargs)
+            host = Host.objects.get(name=splitted[0], domain__name=splitted[1], **kwargs)
         except Host.DoesNotExist:
             return None
         except Host.MultipleObjectsReturned:
@@ -285,6 +290,68 @@ def pre_delete_host(sender, **kwargs):
 pre_delete.connect(pre_delete_host, sender=Host)
 
 
+class RelatedHost(models.Model):
+    # host addr = network_of_main_host + interface_id
+    name = models.CharField(
+        max_length=255,  # RFC 2181 (and considering having multiple joined labels here later)
+        validators=[
+            RegexValidator(
+                regex=r'^(([a-z0-9][a-z0-9\-]*[a-z0-9])|[a-z0-9])$',
+                message='Invalid host name: only "a-z", "0-9" and "-" is allowed'
+            ),
+        ],
+        help_text=_("The name of a host in same network as your main host."))
+    comment = models.CharField(
+        max_length=255,  # should be enough
+        default='', blank=True, null=True,
+        help_text=_("Some arbitrary comment about your host, e.g  who / what / where this host is"))
+    interface_id_ipv4 = models.CharField(
+        default='',
+        max_length=16,  # 123.123.123.123
+        help_text=_("The IPv4 interface ID of this host. Use IPv4 notation."))
+    interface_id_ipv6 = models.CharField(
+        default='',
+        max_length=22,  # ::1234:5678:9abc:def0
+        help_text=_("The IPv6 interface ID of this host. Use IPv6 notation."))
+    available = models.BooleanField(
+        default=True,
+        help_text=_("Check if host is available/in use - "
+                    "if not checked, we won't accept updates for this host"))
+
+    main_host = models.ForeignKey(Host, on_delete=models.CASCADE, related_name='relatedhosts')
+
+    def __unicode__(self):
+        return u"%s.%s" % (
+            self.name, self.main_host.__unicode__())
+
+    class Meta(object):
+        unique_together = (('name', 'main_host'), )
+
+    def get_fqdn(self):
+        main = self.main_host.get_fqdn()
+        # note: we put the related hosts (subhosts) into same zone as the main host,
+        # so the resulting hostname has a dot inside:
+        return dnstools.FQDN('%s.%s' % (self.name, main.host), main.domain)
+
+    def get_ip(self, kind):
+        record = 'A' if kind == 'ipv4' else 'AAAA'
+        try:
+            return dnstools.query_ns(self.get_fqdn(), record)
+        except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
+            return 'none'
+        except (dns.resolver.NoNameservers, dns.resolver.Timeout, dnstools.NameServerNotAvailable):
+            return 'error'
+
+    def get_ipv4(self):
+        return self.get_ip('ipv4')
+
+    def get_ipv6(self):
+        return self.get_ip('ipv6')
+
+
+pre_delete.connect(pre_delete_host, sender=RelatedHost)
+
+
 class ServiceUpdater(models.Model):
     name = models.CharField(
         max_length=32,
