@@ -11,8 +11,8 @@ from nsupdate.main.models import Host
 def test_faults_reset():
     # trigger execution of all code for coverage, test resetting
     # set flags and counters
-    hostname = TEST_HOST.split('.', 1)[0]
-    h = Host.objects.get(subdomain=hostname)
+    hostname = TEST_HOST.host
+    h = Host.objects.get(name=hostname)
     h.client_faults = 1
     h.server_faults = 1
     h.available = False
@@ -26,7 +26,7 @@ def test_faults_reset():
                             reset_abuse=True, reset_abuse_blocked=True,
                             reset_available=True)
     # check if the resetting worked
-    h = Host.objects.get(subdomain=hostname)
+    h = Host.objects.get(name=hostname)
     assert h.client_faults == 0
     assert h.server_faults == 0
     assert h.available is True
@@ -35,28 +35,28 @@ def test_faults_reset():
 
 
 def test_faults_no_abuse():
-    hostname = TEST_HOST.split('.', 1)[0]
-    h = Host.objects.get(subdomain=hostname)
+    hostname = TEST_HOST.host
+    h = Host.objects.get(name=hostname)
     h.client_faults = 10  # below threshold
     h.abuse = False
     h.save()
     # flag abusive hosts
     management.call_command('faults', flag_abuse=23)
     # check if it did not get flagged
-    h = Host.objects.get(subdomain=hostname)
+    h = Host.objects.get(name=hostname)
     assert h.client_faults == 10
     assert h.abuse is False
 
 
 def test_faults_abuse():
-    hostname = TEST_HOST.split('.', 1)[0]
-    h = Host.objects.get(subdomain=hostname)
+    hostname = TEST_HOST.host
+    h = Host.objects.get(name=hostname)
     h.client_faults = 42  # above threshold
     h.abuse = False
     h.save()
     # flag abusive hosts
     management.call_command('faults', flag_abuse=23)
     # check if it did get flagged
-    h = Host.objects.get(subdomain=hostname)
+    h = Host.objects.get(name=hostname)
     assert h.client_faults == 0
     assert h.abuse is True
